/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2008 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef EXCEPTIONS_H
#define EXCEPTIONS_H

#include <iostream>
#include <string>
#include <exception>

using std::string;
using std::exception;

/**
 * An U2D_IO_Exception is generated whenever a file
 * input output operation fails.
 * @author Gabriele Dini Ciacci <dark.schneider@iol.it>
 */
class u2d_IO_Exception : public exception {
public:
    virtual const char* what() const throw ();
    u2d_IO_Exception(string reason);
    ~u2d_IO_Exception() throw();
private:
    string my_reason;
};

/**
 * An U2D_Parse_Error_Exception is generated whenever a
 * string parse fails.
 * @author Gabriele Dini Ciacci <dark.schneider@iol.it>
 */
class  u2d_Parse_Error_Exception : public exception
{
public:
    virtual const char* what() const throw ();
     u2d_Parse_Error_Exception(string reason);
    ~ u2d_Parse_Error_Exception() throw();
private:
    string my_reason;
};

/**
 * An U2D_Size_Missmatch_Exception is generated whenever a
 * vector size or matrix size is not what expected.
 * @author Gabriele Dini Ciacci <dark.schneider@iol.it>
 */
class  u2d_Size_Missmatch_Exception : public exception
{
public:
    virtual const char* what() const throw ();
     u2d_Size_Missmatch_Exception(string reason);
    ~ u2d_Size_Missmatch_Exception() throw();
private:
    string my_reason;
};

/**
 * An U2D_Math_Exception is generated whenever a
 * problem with a mathematical condition if hit.
 * @author Gabriele Dini Ciacci <dark.schneider@iol.it>
 */
class  u2d_Math_Exception : public exception
{
public:
    virtual const char* what() const throw ();
     u2d_Math_Exception(string reason);
    ~ u2d_Math_Exception() throw();
private:
    string my_reason;
};

/**
 * An U2D_OpenGL_Exception is generated whenever a
 * problem with the local openGL implementation is hit.
 * @author Gabriele Dini Ciacci <dark.schneider@iol.it>
 */
class  u2d_OpenGL_Exception : public exception
{
public:
    virtual const char* what() const throw ();
     u2d_OpenGL_Exception(string reason);
    ~ u2d_OpenGL_Exception() throw();
private:
    string my_reason;
};
#endif //exceptions_h
