/*

    umesh2dgui, a GUI for umesh2d grid generator

    Copyright (C) 2010 Gabriele Dini Ciacci <dark.schneider@iol.it>
    Based on a code from Luigi Quartapelle and Massimo Biava

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Created on January 14, 2010, 12:31 PM
 */


#ifndef U2D_GAUSSPOINTS_H
#define	U2D_GAUSSPOINTS_H

#include "u2d_Mesh.h"
#include "boost/numeric/mtl/mtl.hpp"

using namespace mtl;

/**
 * Calculates the gauss points for p1 on triangles with constant
 * derivatives over each finite element and therefore with derivative
 * and jacobian values independent of the gauss points.
 */
class u2d_DomainGaussPoints {
public:
    u2d_DomainGaussPoints(int _domain_size);
    u2d_DomainGaussPoints(const u2d_DomainGaussPoints& orig);
    virtual ~u2d_DomainGaussPoints();

    /* FROM DOMAIN */
    dense2D<double> dw_x;
    dense2D<double> dw_y;
    dense_vector<double> jacobian_weight;
private:
    int domain_size;

};

/**
 * Calculates the gauss points for p1 on triangles with constant
 * derivatives over each finite element and therefore with derivative
 * and jacobian values independent of the gauss points.
 */
class u2d_BoundaryGaussPoints {
public:
    u2d_BoundaryGaussPoints(int _boundary_size);
    u2d_BoundaryGaussPoints(const u2d_BoundaryGaussPoints& orig);
    virtual ~u2d_BoundaryGaussPoints();


    /* FROM BOUNDARY */
    dense_vector<double> norm_x;
    dense_vector<double> norm_y;
    dense2D<double> dw_x;
    dense2D<double> dwp_x;
    dense_vector<double> jacobian_weight;

private:
    int boundary_size;

};


#endif	/* U2D_GAUSSPOINTS_H */


