/* 
 * File:   u2d_IsoElement1DP12Points.cpp
 * Author: Gabriele Dini Ciacci <dark.schneider@iol.it>
 * 
 * Created on January 17, 2010, 8:40 PM
 */

#include <math.h>

#include "u2d_IsoElement1DP12Points.h"
#include "u2d_GaussPoints.h"

u2d_IsoElement1DP12Points::u2d_IsoElement1DP12Points() :
gauss_points_n(2),
gauss_points(1, gauss_points_n),
w(P1_1D_DEGREE_FREEDOM, gauss_points_n),
dx(P1_1D_DEGREE_FREEDOM, gauss_points_n),
p(gauss_points_n) {

    gauss_points(0, 0) = -1.0 / sqrt(3.0);

    gauss_points(0, 1) = +1.0 / sqrt(3.0);

    for (int i = 0; i < gauss_points_n; i++) {
        w(0, i) = shapeP1Node1(gauss_points(0, i));
        dx(0, i) = -1.0 / 2.0;

        w(1, i) = shapeP1Node2(gauss_points(0, i));
        dx(1, i) = 1.0 / 2.0;

        p(i) = 1.0;
    }
}

u2d_IsoElement1DP12Points::u2d_IsoElement1DP12Points(const u2d_IsoElement1DP12Points& orig) {
}

u2d_IsoElement1DP12Points::~u2d_IsoElement1DP12Points() {
}

/**
 * Evaluate and store the values of derivatives and of the
 * jacobian determinant at Gauss points of all boundary elements.
 */
u2d_BoundaryGaussPoints *u2d_IsoElement1DP12Points::applyTo(u2d_Mesh *mesh) {

    u2d_BoundaryGaussPoints *bgp;
    bgp = new u2d_BoundaryGaussPoints(mesh->getBoundaryElementNumber());

    /* For the linear element the derivatives are constant and
     * the weights are uniform so the first Gauss point is used
     */
    int gauss_point = 0; /* The first */

    /* We calculate dd_x vector for the first gauss
     * point outside the element loop, because they are the
     * same for all the elements, since they depend on the isoelement only
     */
    dense_vector<double> dd_x(BOUNDARY_ELEMENT_SIZE);
    for (int ds = 0; ds < BOUNDARY_ELEMENT_SIZE; ds++) {
        dd_x(ds) = dx(ds, gauss_point);
    }

    for (int i = 0; i < mesh->getBoundaryElementNumber(); i++) {

        multi_vector<dense_vector<double> > elem_i = mesh->getBoundaryElementNodes(i + 1);
        /* Assemble the jacobian */
        dense2D<double> dr(1, 2);
        dr(0, 0) = dot(elem_i.vector(0), dd_x);
        dr(0, 1) = dot(elem_i.vector(1), dd_x);

        /* Calculate the jacobian determinant */
        double jacobian = sqrt(pow(dr(0, 0), 2.0) + pow(dr(0, 1), 2.0));

        /* Calculare the norms */
        bgp->norm_x(i) = dr(0, 1) / jacobian; // outward normal
        bgp->norm_y(i) = -dr(0, 0) / jacobian; // outward normal

        bgp->jacobian_weight(i) = p(gauss_point) * jacobian;

        /* Necessary only for evaluating gradient
         * tangential to the surface (ex COMMON Gauss_tan)
         */
        bgp->dw_x(0, i) = dd_x(0);
        bgp->dw_x(1, i) = dd_x(1);
        bgp->dwp_x(0, i) = dd_x(0) * p(gauss_point);
        bgp->dwp_x(1, i) = dd_x(1) * p(gauss_point);
    }

    // Lovely debug output
    //std::cout << bgp->dw_x << std::endl;
    //std::cout << bgp->dwp_x << std::endl;
    //std::cout << norm_x << std::endl;
    //std::cout << norm_y << std::endl;
    //std::cout << jacobian_weight << std::endl;
    return bgp;
}