!     generation of a square grid of equal triangular elements
!     over the unit square
!     no triangle has two sides on the boundary

      PROGRAM gen_sq_grid

      IMPLICIT NONE

      INTEGER, PARAMETER :: n_w = 3,  n_ws = 2

      REAL(KIND=8), DIMENSION(:,:), ALLOCATABLE :: rr
      INTEGER,      DIMENSION(:,:), ALLOCATABLE :: jj, neigh, jjs, iis
      INTEGER,      DIMENSION(:),   ALLOCATABLE :: m_a, sides

!     neigh(k, m)  = number of element opposite to node  k  of element  m
!     m_a(ms)      = number of element adiacent to surface element  ms

      INTEGER      :: lx, ly, np, me, mes,  &
                      n, m, j, ja, i, ms, i_z
    
      REAL(KIND=8) :: dx, dy


!      PRINT *, ' lx  and  ly ?'
      READ (*,*)  lx, ly

      np  = (lx + 1) * (ly + 1)
      me  = 2 * lx * ly
      mes = 2 * lx + 2 * ly

      ALLOCATE (rr(2, np),  jj(n_w,  me),  neigh(n_w, me),   &
                           jjs(n_ws, mes),  m_a(mes),  sides(mes),   &
                           iis(n_ws, mes))

!     /* topology of volume elements */

      m = 0
      DO j = 1, ly
         ja = (j-1) * (lx+1)
         DO i = 1, lx
            m = m + 1
            jj(1, m) = ja + i
            jj(2, m) = ja + i + 1
            jj(3, m) = ja + i + (lx+1)
            m = m + 1
            jj(1, m) = ja + i + (lx+1) + 1
            jj(2, m) = ja + i + (lx+1)
            jj(3, m) = ja + i + 1
         ENDDO
      ENDDO


!     correction for the first and the last couples of elements

      m = 1
      jj(1, m) = 1
      jj(2, m) = 2
      jj(3, m) = (lx+1) + 2
      m = 2
      jj(1, m) = (lx+1) + 2
      jj(2, m) = (lx+1) + 1
      jj(3, m) = 1

      m = me - 1
      jj(1, m) = ly * (lx+1) - 1
      jj(2, m) = ly * (lx+1)
      jj(3, m) = np
      m = me
      jj(1, m) = np
      jj(2, m) = np - 1
      jj(3, m) = ly * (lx+1) - 1


!     determine the elements opposite to the nodes of each element

      m = 0
      DO j = 1, ly
         DO i = 1, lx
            m = m + 1
            neigh(1, m) = m + 1
            neigh(2, m) = m - 1
            neigh(3, m) = (j-2) * 2 * lx + 2 * i
            m = m + 1
            neigh(1, m) = m - 1
            neigh(2, m) = m + 1
            neigh(3, m) = j * 2 * lx + 2 * i - 1
         ENDDO
      ENDDO

      DO i = 1, lx
         m = 2 * i - 1
         neigh(3, m) = 0
         m = (ly - 1) * 2 * lx + 2 * i
         neigh(3, m) = 0
      ENDDO

      DO j = 1, ly
          m = (j - 1) * 2 * lx + 1
          neigh(2, m) = 0
          m = j * 2 * lx
          neigh(2, m) = 0
      ENDDO


!     correction for the first couple of elements at bottom left corner 
!     and for the last couple of elements at the top right corner

      neigh(1, 1) = 3
      neigh(2, 1) = 2
      neigh(3, 1) = 0

      neigh(1, 2) = 0
      neigh(2, 2) = 1
      neigh(3, 2) = 2 * lx + 1

      neigh(2, 3) = 1

      m = me - 1
      neigh(1, m) = 0
      neigh(2, m) = me
      neigh(3, m) = (ly - 1) * 2 * lx

      m = me
      neigh(1, m) = me - 2
      neigh(2, m) = me - 1
      neigh(3, m) = 0

      m = me - 2
      neigh(2, m) = me



!     /* topology of surface elements */

      ms = 0

      DO i = 1, lx  !  bottom side # 1
         ms = ms + 1
         m_a(ms) = 2 * i - 1
         jjs(1, ms) = i
         jjs(2, ms) = i + 1
         sides(ms) = 1
      ENDDO

      DO j = 1, ly  !  left side # 2
         ms = ms + 1
         m_a(ms) = j * 2 * lx
         jjs(1, ms) =   j   * (lx+1)
         jjs(2, ms) = (j+1) * (lx+1)
         sides(ms) = 2
      ENDDO

      m_a(ms) = ly * 2 * lx - 1 ! correction for last couple

      DO i = lx, 1, -1  !  top side # 3
         ms = ms + 1
         m_a(ms) = (ly-1) * 2 * lx + 2 * i
         jjs(1, ms) = ly * (lx+1) + i + 1
         jjs(2, ms) = ly * (lx+1) + i
         sides(ms) = 3
      ENDDO

      DO j = ly, 1, -1  !  right side # 4
         ms = ms + 1
         m_a(ms) = (j-1) * 2 * lx + 1
         jjs(1, ms) =   j   * (lx+1) + 1
         jjs(2, ms) = (j-1) * (lx+1) + 1
         sides(ms) = 4
      ENDDO
    
      m_a(ms) = 2 ! correction for first couple


!     /* nodal coordinates */

      dx = 1.0d0/lx
      dy = 1.0d0/ly
      n = 0
      DO j = 1, ly+1
         DO i = 1, lx+1
            n = n + 1
            rr(1, n) = (i-1) * dx
            rr(2, n) = (j-1) * dy
         ENDDO
      ENDDO


!     write the grid in the standard format

      WRITE (*,100) np, n_w, me, n_ws, mes

      i_z = 0

      DO m = 1, me
         WRITE (*,100)   jj(1,m),    jj(2,m),    jj(3,m),  i_z,  & 
                      neigh(1,m), neigh(2,m), neigh(3,m)
      ENDDO


      DO ms = 1, mes
         WRITE (*,100) m_a(ms), jjs(1,ms), jjs(2,ms), sides(ms)
      ENDDO

      DO n = 1, np
         WRITE (*,200) rr(1,n), rr(2,n)
      ENDDO

!     do m = 1, me
!        m1 = neigh(1, m)
!        m2 = neigh(2, m)
!        m3 = neigh(3, m)
!        write (*,100) m, m1, m2, m3
!     enddo
!
!     write (*,*) m
!
!     do m = 1, me
!        n1 = jj(1, m)
!        n2 = jj(2, m)
!        n3 = jj(3, m)
!        write (*,100) m, n1, n2, n3
!     enddo

!      print*, 'end of grid writing'

 100  FORMAT (8i10)
 200  FORMAT (4d24.17)

! SCRIVERE DUE FILES:  grid.nome  e  nodes.nome


      END PROGRAM gen_sq_grid
